/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Sonatype,Nexus*/
define('Sonatype/repoServer/SmartProxySettings', function() {

  Sonatype.repoServer.SmartProxySettings = function (config) {
      config = config || {};

      var defaultConfig, deleteButton, editButton;

      defaultConfig = {
          region: 'center',
          width: '100%',
          height: '100%',
          autoScroll: true,
          dataModifiers: {
              load: {
                  // 'rootData' : this.loadFingerprint.createDelegate(this)
              },
              trustedKeys: this.updateTrustedKeys.createDelegate(this)
          }
      };

      Ext.apply(this, config, defaultConfig);

      this.trustedKeysRecordConstructor = Ext.data.Record.create([
          {
              name: 'id'
          },
          {
              name: 'certificate'
          },
          {
              name: 'description',
              sortType: Ext.data.SortTypes.asUCString
          }
      ]);

      this.trustedKeysReader = new Ext.data.JsonReader({
          root: 'data',
          id: 'id'
      }, this.trustedKeysRecordConstructor);

      this.trustedKeysStore = new Ext.data.Store({
          url: Sonatype.config.servicePath + '/smartproxy/trusted-keys',
          reader: this.trustedKeysReader,
          sortInfo: {
              field: 'description',
              direction: 'ASC'
          },
          autoLoad: true,
          listeners: {
              loadexception: {
                  fn: function () {
                      this.trustedKeysStore.removeAll();
                  },
                  scope: this
              }
          }
      });

      this.referenceData = {
          enabled: false,
          host: '',
          port: 0,
          advertiseUri: ''
      };

      deleteButton = new Ext.Button({
          text: 'Delete',
          cls: 'x-btn-text-icon',
          icon: Sonatype.config.resourcePath + '/static/images/icons/delete.png',
          disabled: true,
          handler: this.removeSelectedPublicKey,
          scope: this
      });

      editButton = new Ext.Button({
          text: 'Edit',
          cls: 'x-btn-text-icon',
          icon: Sonatype.config.resourcePath + '/static/images/icons/page_white_put.png',
          disabled: true,
          handler: this.editSelectedPublicKey,
          scope: this
      });

      this.networkForm = new Ext.FormPanel({
          labelWidth: 100,
          region: 'center',
          layoutConfig: {
              labelSeparator: ''
          },
          items: [
              {
                  xtype: 'fieldset',
                  checkboxToggle: false,
                  title: 'Network Settings',
                  anchor: Sonatype.view.FIELDSET_OFFSET,
                  collapsible: false,
                  autoHeight: true,
                  buttonAlign: 'center',
                  items: [
                      {
                          xtype: 'checkbox',
                          fieldLabel: 'Enable Server',
                          name: 'enabled',
                          anchor: Sonatype.view.FIELD_OFFSET
                      },
                      {
                          xtype: 'textfield',
                          fieldLabel: 'Address',
                          helpText: 'Configures the address which the connector will bind to.  Defaults to 0.0.0.0 to bind to all addresses.',
                          name: 'host',
                          emptyText: '0.0.0.0',
                          anchor: Sonatype.view.FIELD_OFFSET
                      },
                      {
                          xtype: 'numberfield',
                          fieldLabel: 'Port',
                          helpText: 'Configures the port which the connector which will listen for client connections; A value of 0 will use a random port.',
                          name: 'port',
                          validator: function (port) {
                              if (port >= 0 && port <= 65535) {
                                  return true;
                              }
                              else {
                                  return 'Invalid port number (0 for random, positive and less than or equal to 65535 otherwise)';
                              }
                          },
                          allowNegative: false,
                          allowDecimals: false,
                          emptyText: '0',
                          anchor: Sonatype.view.FIELD_OFFSET
                      },
                      {
                          xtype: 'textfield',
                          fieldLabel: 'Advertised URI',
                          helpText: 'URI which will be advertised to clients for establishing connections. Defaults to the URI reported by the connector.',
                          name: 'advertiseUri',
                          anchor: Sonatype.view.FIELD_OFFSET,
                          emptyText: ''
                      },
                      {
                          xtype: 'fieldset',
                          id: 'sp_settings_status_fieldset',
                          checkboxToggle: false,
                          title: 'Status',
                          anchor: Sonatype.view.FIELDSET_OFFSET,
                          collapsible: false,
                          autoHeight: true,
                          layoutConfig: {
                              labelSeparator: ''
                          },
                          items: [
                              {
                                  fieldLabel: 'Status',
                                  hideLabel: true,
                                  name: 'status',
                                  anchor: Sonatype.view.FIELDSET_OFFSET,
                                  xtype: 'displayfield',
                                  width: 700
                              }
                          ]
                      }
                  ],
                  buttons: [
                      {
                          text: 'Save',
                          handler: this.saveNetworkSettings,
                          scope: this
                      },
                      {
                          handler: this.loadNetworkSettings,
                          scope: this,
                          text: 'Cancel'
                      }
                  ]
              },
              {
                  xtype: 'fieldset',
                  checkboxToggle: false,
                  title: 'Public Key',
                  collapsible: false,
                  autoHeight: true,
                  layoutConfig: {
                      labelSeparator: ''
                  },
                  anchor: Sonatype.view.FIELDSET_OFFSET,
                  items: [
                      {
                          xtype: 'textarea',
                          fieldLabel: 'Certificate',
                          name: 'identity.pem',
                          helpText: 'The public key certificate for this server. Install this on remote Nexus instances to allow this server to access remotes Smart Proxy services.',
                          readOnly: true,
                          height: 150,
                          anchor: Sonatype.view.FIELDSET_OFFSET,
                          listeners: {
                              render: function (component) {
                                  component.getEl().on('click', function () {
                                      var pos = component.getEl().dom.scrollTop;
                                      component.selectText();
                                      setTimeout(function () {
                                          component.getEl().dom.scrollTop = pos;
                                      }, 50);
                                  });
                              }
                          }
                      },
                      {
                          fieldLabel: 'Fingerprint',
                          xtype: 'displayfield',
                          name: 'identity.fingerprint',
                          anchor: Sonatype.view.FIELDSET_OFFSET
                      }
                  ]
              }
          ]
      });

      this.networkForm.on('actioncomplete',
          function (form, action) {
              var enabled, showStatus, statusFieldSet;

              enabled = this.networkForm.getForm().findField('enabled');

              showStatus = enabled && enabled.getValue();

              if (showStatus) {
                  if (action.type == 'sonatypeSubmit') {
                      // reload form to get enabled info
                      this.loadNetworkSettings();
                  }
              }

              statusFieldSet = Ext.getCmp('sp_settings_status_fieldset');
              if (statusFieldSet !== undefined) {
                  statusFieldSet.setVisible(showStatus);
              }

          }.createDelegate(this));

      Sonatype.repoServer.SmartProxySettings.superclass.constructor.call(this, {
          trackResetOnLoad: true,
          border: false,
          frame: true,
          // scrolling is not enabled correctly, so we have to force the CSS style
          bodyStyle: 'overflow: auto',
          labelWidth: 150,
          layoutConfig: {
              labelSeparator: ''
          },
          items: [
              this.networkForm, new Ext.FormPanel({
                      labelWidth: 100,
                      region: 'center',
                      layoutConfig: {
                          labelSeparator: ''
                      },
                      items: [
                          {
                              xtype: 'fieldset',
                              checkboxToggle: false,
                              title: 'Trusted Certificates',
                              anchor: Sonatype.view.FIELDSET_OFFSET,
                              autoHeight: true,
                              collapsible: false,
                              layoutConfig: {
                                  labelSeparator: ''
                              },
                              items: [
                                  {
                                      xtype: 'grid',
                                      name: 'trustedKeys',
                                      helpText: 'The public key certificates which have been installed on this server. Remote Nexus instances in this list can access Smart Proxy services.',
                                      anchor: Sonatype.view.FIELD_OFFSET,
                                      ds: this.trustedKeysStore,
                                      height: 250,
                                      viewConfig: {
                                          forceFit: true
                                      },
                                      listeners: {
                                          'celldblclick': {
                                              fn: function (grid, rowIdx, columnIdx, evt) {
                                                  this.editSelectedPublicKey();
                                              },
                                              scope: this
                                          }
                                      },
                                      sm: new Ext.grid.RowSelectionModel({
                                          singleSelect: true,
                                          listeners: {
                                              rowselect: function () {
                                                  editButton.enable();
                                                  deleteButton.enable();
                                              },
                                              rowdeselect: function () {
                                                  editButton.disable();
                                                  deleteButton.disable();
                                              },
                                              scope: this
                                          }

                                      }),
                                      columns: [
                                          {
                                              id: 'certificate',
                                              header: "Fingerprint",
                                              sortable: true,
                                              dataIndex: 'certificate',
                                              renderer: this.fingerprintRenderer
                                          },
                                          {
                                              id: 'description',
                                              header: "Description",
                                              width: 300,
                                              sortable: true,
                                              dataIndex: 'description'
                                          }
                                      ]
                                  }
                              ],
                              tbar: [
                                  {
                                      text: 'Refresh',
                                      cls: 'x-btn-text-icon',
                                      icon: Sonatype.config.resourcePath + '/static/images/icons/arrow_refresh.png',
                                      handler: this.refreshKeys,
                                      scope: this
                                  },
                                  {
                                      text: 'Add',
                                      cls: 'x-btn-text-icon',
                                      icon: Sonatype.config.resourcePath + '/static/images/icons/add.png',
                                      handler: function () {
                                          this.showAddCertificateWindow('Add Trusted Certificate', undefined);
                                      },
                                      scope: this
                                  },
                                  editButton,
                                  deleteButton
                              ]}
                      ]}
              )
          ]
      });

      this.on('afterlayout', this.loadNetworkSettings, this, {
          single: true
      });

  };

  Ext.extend(Sonatype.repoServer.SmartProxySettings, Ext.Panel, Ext.applyIf({
      loadNetworkSettings: function () {
          this.networkForm.getForm().doAction('sonatypeLoad', {
              fpanel: this.networkForm,
              method: 'GET',
              scope: this,
              url: Sonatype.config.servicePath + '/smartproxy/settings'
          });
      },
      saveNetworkSettings: function () {
          this.networkForm.getForm().doAction('sonatypeSubmit', {
              method: 'PUT',
              fpanel: this.networkForm,
              serviceDataObj: this.referenceData,
              scope: this,
              waitMsg: 'Saving network settings...',
              dataModifiers: {
                  port: function (value) {
                      return value ? value : 0;
                  }
              },
              url: Sonatype.config.servicePath + '/smartproxy/settings'
          });
      },

      fingerprintRenderer: function (value, metaData, record, rowIndex, colIndex, store) {
          return value.fingerprint;
      },

      refreshKeys: function () {
          this.trustedKeysStore.reload();
      },

      removeSelectedPublicKey: function () {
          var sm, rec;

          sm = this.find('name', 'trustedKeys')[0].getSelectionModel();
          rec = sm.getSelected();

          Sonatype.MessageBox.show({
              title: 'Delete Certificate?',
              msg: 'Delete trusted certificate: ' + rec.get('description') + ' ?',
              buttons: Sonatype.MessageBox.YESNO,
              scope: this,
              icon: Sonatype.MessageBox.QUESTION,
              fn: function (btnName) {
                  if (btnName === 'yes') {
                      Ext.Ajax.request({
                          url: Sonatype.config.servicePath + '/smartproxy/trusted-keys/' + rec.get('id'),
                          method: 'DELETE',
                          callback: function () {
                              sm.clearSelections();
                              this.trustedKeysStore.remove(rec);
                          },
                          scope: this
                      });
                  }
              }
          });
      },

      editSelectedPublicKey: function () {
          var rec = this.find('name', 'trustedKeys')[0].getSelectionModel().getSelected();

          this.showAddCertificateWindow('Trusted Certificate', rec);
      },

      updateTrustedKeys: function (val, panel) {
          var data, items, i;
          data = [];
          items = this.trustedKeysStore.data.items;
          for (i = 0; i < items.length; i++) {
              data.push(items[i].data);
          }
          return data;
      }

  }, Sonatype.repoServer.SmartProxyCertificateUtils));

  Sonatype.Events.addListener('nexusNavigationInit', function (nexusPanel) {
      if (Sonatype.lib.Permissions.checkPermission('nexus:smartproxy:settings', Sonatype.lib.Permissions.READ)) {
          nexusPanel.add({
              enabled: true,
              sectionId: 'st-nexus-enterprise',
              title: 'Smart Proxy',
              tabId: 'smart_proxy',
              tabTitle: 'Smart Proxy',
              tabCode: Sonatype.repoServer.SmartProxySettings
          });
      }
  });
});

